/* TroiaPreview
 * version 1.0 date 2005-03-24
 * Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <string.h>
#include <winsock.h>

typedef unsigned short uint16_t;
typedef unsigned int in_addr_t;

#include "Globals.h"
#include "Pixel.h"
#include "Frame.h"
#include "StreamRecv.h"

//private constants
#define StreamRecvRequestInterval 10.0

//private global variables
struct sockaddr_in StreamRecvBindAddr; //address to bind socket to
int StreamRecvSocket = -1; //socket for network communication: none yet
int StreamRecvProxy = 0; //if to use blinkenproxy
struct sockaddr_in StreamRecvProxyAddr; //address of blinkenproxy
double StreamRecvRequestLast; //last time request was sent to blinkenproxy

//start stream reception animation
//BindAddr must be of format 0.0.0.0:2323
//ProxyAddr must be of format 0.0.0.0:4242 or NULL
void StreamRecvStart( char * BindAddr, char * ProxyAddr )
{
  char * p_colon;
  int port, yes;
  in_addr_t addr;

  //stop stream reception
  StreamRecvStop( );

  //parse bind address
  p_colon = strchr( BindAddr, ':' );
  if( p_colon == NULL )
  {
    if( sscanf( BindAddr, "%d", &port ) != 1 || port < 0 || port > 65535 )
      port = 2323;
    addr = INADDR_ANY;
  }
  else
  {
    if( sscanf( p_colon+1, "%d", &port ) != 1 || port < 0 || port > 65535 )
      port = 2323;
    *p_colon = 0;
    addr = inet_addr( BindAddr );
    *p_colon = ':';
    if( addr == INADDR_NONE )
      addr = INADDR_ANY;
  }
  //save bind address
  StreamRecvBindAddr.sin_family = AF_INET;
  StreamRecvBindAddr.sin_port = htons( (uint16_t)port );
  StreamRecvBindAddr.sin_addr.s_addr = addr;

  //use blinkenproxy?
  StreamRecvProxy = ProxyAddr != NULL;
  if( StreamRecvProxy )
  {
    //parse proxy address
    p_colon = strchr( ProxyAddr, ':' );
    if( p_colon == NULL )
    {
      if( sscanf( ProxyAddr, "%d", &port ) != 1 || port < 0 || port > 65535 )
        port = 4242;
      addr = INADDR_ANY;
    }
    else
    {
      if( sscanf( p_colon+1, "%d", &port ) != 1 || port < 0 || port > 65535 )
        port = 4242;
      *p_colon = 0;
      addr = inet_addr( ProxyAddr );
      *p_colon = ':';
      if( addr == INADDR_NONE )
        addr = INADDR_ANY;
    }
    //save proxy address
    StreamRecvProxyAddr.sin_family = AF_INET;
    StreamRecvProxyAddr.sin_port = htons( (uint16_t)port );
    StreamRecvProxyAddr.sin_addr.s_addr = addr;
  }

  //create socket
  StreamRecvSocket = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );
  if( StreamRecvSocket == -1 )
    return;

  //unblock socket
  yes = 1;
  if( ioctlsocket( StreamRecvSocket, FIONBIO, &yes ) < 0 )
  {
    closesocket( StreamRecvSocket );
    StreamRecvSocket = -1;
    return;
  }
                                       
  //bind socket
  if( bind( StreamRecvSocket, (struct sockaddr *)&StreamRecvBindAddr, sizeof( StreamRecvBindAddr ) ) < 0 )
  {
    closesocket( StreamRecvSocket );
    StreamRecvSocket = -1;
    return;
  }

  //perhaps send blinkenproxy request
  if( StreamRecvProxy )
  {
    sendto( StreamRecvSocket, "\x42\x42\x42\x42\0\0\0\0\0\0\0\0", 12, 0,
            (struct sockaddr *)&StreamRecvProxyAddr, sizeof( StreamRecvProxyAddr ) );
    StreamRecvRequestLast = TimeGet( );
  }
}

//stop stream reception
void StreamRecvStop( )
{
  //exit if stream reception not running
  if( StreamRecvSocket == -1 )
    return;

  //perhaps cancel blinkenproxy
  if( StreamRecvProxy )
    sendto( StreamRecvSocket, "\x42\x42\x42\x43\0\0\0\0\0\0\0\0", 12, 0,
            (struct sockaddr *)&StreamRecvProxyAddr, sizeof( StreamRecvProxyAddr ) );

  //close socket
  closesocket( StreamRecvSocket );
  StreamRecvSocket = -1;
}

//called periodically to do some work
void StreamRecvTask( )
{
  double CurrentTime;
  int len;
  unsigned char PacketBuffer[65536]; //more than max. sizie of UDP packet

  //exit if stream reception not running
  if( StreamRecvSocket == -1 )
    return;

  //get time
  CurrentTime = TimeGet( );

  //receive stream packets
  for( ; ; )
  {
    //get next packet
    len = recv( StreamRecvSocket, PacketBuffer, sizeof( PacketBuffer ), 0 );
    if( len < 0 ) //no packet available
      break;
    //check length and header
    if( len < 12 || //truncated header
        PacketBuffer[0] != 0x23 || //wrong magic
        PacketBuffer[1] != 0x54 ||
        PacketBuffer[2] != 0x26 ||
        PacketBuffer[3] != 0x66 ||
        ntohs( *(uint16_t *)(PacketBuffer+4) ) != PixelY + PixelZ || //wrong height
        ntohs( *(uint16_t *)(PacketBuffer+6) ) != 2 * PixelX + 2 * PixelY || //wrong width
        ntohs( *(uint16_t *)(PacketBuffer+8) ) != 1 || //wrong number of channels
        ntohs( *(uint16_t *)(PacketBuffer+10) ) != 255 || //wrong maximum greyscale value
        len < 12 + (PixelY + PixelZ) * (2 * PixelX + 2 * PixelY) ) //packet too short
      continue;
    //put data from packet into frame
    FrameSet2D( PacketBuffer+12 );
  }

  //perhaps send blinkenproxy request
  if( StreamRecvProxy &&
      CurrentTime >= StreamRecvRequestLast + StreamRecvRequestInterval )
  {
    sendto( StreamRecvSocket, "\x42\x42\x42\x42\0\0\0\0\0\0\0\0", 12, 0,
            (struct sockaddr *)&StreamRecvProxyAddr, sizeof( StreamRecvProxyAddr ) );
    StreamRecvRequestLast = CurrentTime;
  }
}
