/* TroiaPreview
 * version 1.0 date 2005-03-24
 * Copyright (C) 2003-2005 Stefan Schuermans <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdlib.h>

#include "Globals.h"
#include "Pixel.h"
#include "Frame.h"
#include "Tracking.h"

//private global variables
unsigned char TrackingSize = 0; //tracking not active
unsigned short TrackingForeColor, TrackingBackColor; //colors for tracking cross (256 is transparent)

//draw tracking cross
static void TrackingDraw( )
{
  double X, Y;
  unsigned char I, J, Dir, Dir2;
  tPixel3D TrackPos, Pix, Pix2;

  //get new tracking position
  X = Pos.X / PixelSize - 0.5; //get position in pixels (0.0 is in middle of pixel 0)
  Y = PixelY - Pos.Y / PixelSize - 0.5;
  TrackPos.Wall = 6; //floor
  TrackPos.X = ShortMinMax( (short)(X + 0.5), 0, PixelX - 1 );
  TrackPos.Y = ShortMinMax( (short)(Y + 0.5), 0, PixelY - 1 );

  //draw tracking cross
  if( TrackingForeColor < 256 )
    FrameTmpSetPixel( TrackPos, (unsigned char)TrackingForeColor );
  for( I = 0; I < 4; I++ )
  {
    Pix = TrackPos;
    Dir = DirRotate( 1, I * 2 ); //every diagonal direction
    for( J = 0; J < TrackingSize; J++ )
    {
      Pixel3DMove( Pix, Dir, &Pix, &Dir ); //on diagonal line
      if( J < TrackingSize - 1 )
      {
        if( TrackingForeColor < 256 ) //256 is transparent
          FrameTmpSetPixel( Pix, (unsigned char)TrackingForeColor );
      }
      else
      {
        if( TrackingBackColor < 256 ) //256 is transparent
          FrameTmpSetPixel( Pix, (unsigned char)TrackingBackColor );
      }
      if( TrackingBackColor < 256 ) //256 is transparent
      {
        Pixel3DMove( Pix, DirRotate( Dir, 3 ), &Pix2, &Dir2 ); //next to diagonal line - one side
        FrameTmpSetPixel( Pix2, (unsigned char)TrackingBackColor );
        Pixel3DMove( Pix, DirRotate( Dir, -3 ), &Pix2, &Dir2 ); //next to diagonal line - other side
        FrameTmpSetPixel( Pix2, (unsigned char)TrackingBackColor );
        if( J < TrackingSize - 1 )
        {
          Pixel3DMove( Pix, DirRotate( Dir, 2 ), &Pix2, &Dir2 ); //diagonally next to diagonal line - one side
          FrameTmpSetPixel( Pix2, (unsigned char)TrackingBackColor );
          Pixel3DMove( Pix, DirRotate( Dir, -2 ), &Pix2, &Dir2 ); //diagonally next to diagonal line - other side
          FrameTmpSetPixel( Pix2, (unsigned char)TrackingBackColor );
        }
      }
    } //for( J ...
  } //for( I ...
}

//start tracking
//color 256 is transparent
void TrackingStart( unsigned char Size, unsigned short ForeColor, unsigned short BackColor )
{
  TrackingSize = CharMinMax( Size, TrackingMinSize, TrackingMaxSize );
  TrackingForeColor = ForeColor;
  TrackingBackColor = BackColor;
  TrackingDraw( );
}

//stop tracking
void TrackingStop( )
{
  TrackingSize = 0;
}

//called periodically to do some work
void TrackingTask( )
{
  //exit if tracking not running
  if( TrackingSize == 0 )
    return;

  //draw tracking cross
  TrackingDraw( );
}
